// Get categories from dictionary passed by submit process
const CATS = submitContext.dictionaries.categories

// Check if user is authorized
async function checkAuth() {
  // [...]
  return true;
}

// Get list of not submitted files
async function getPendingFiles() {
  // [...] 
  return []
}

// Find uploaded files on microstock
async function findAssets(assets) {
   
    // Get list of not submitted files
    const pendingFiles = await getPendingFiles();
    
    // Find matches
    const foundPairs = new Map();
    for (const pendingFile of pendingFiles){
        for (const asset of assets){
            if (foundPairs.has(asset)){
               continue; // Match already found for this file
            }
            
            // Check, if file names (without extensions) are same
            if (pendingFile.basename === asset.uploadedBasename){
                foundPairs.set(asset, pendingFile)
            }
        }
    }
    
    // Additionally check the list of failed uploads if possible
    // [...]
   
    // Return matched files
    return [...foundPairs.entries()].map(([asset, stockFile]) => {
        return {
            asset,
            stockFile
        }
    })
}

// Find already uploaded release
async function findReleaseOnStock(releaseAsset) { 
    // [...] 
    return null
}

const ReleasesMap = new Map(); // Releases cache
async function uploadReleases(foundAssets) {
    for (const { asset } of foundAssets){
        // Check, if file has attached releases
        if (asset.metadata.releases && asset.metadata.releases.length > 0){
            for (const releaseLink of asset.metadata.releases){
                if (ReleasesMap.has(releaseLink.assetId)){
                    continue;
                }
                // Read information about release
                const releaseAsset = await window.imshost.loadAsset(releaseLink.assetId);
                
                // Find release in list of already uploaded releases
                const stockReleaseId = await findReleaseOnStock(releaseAsset);
                
                if (stockReleaseId){
                    ReleasesMap.set(releaseLink.assetId, stockReleaseId);
                }
                else {
                    // Get main file of release
                    const releaseMainFile = releaseAsset.mainFile;
                    
                    // Read contents of main file
                    const blob = await releaseMainFile.getBlob();
                    
                    // Upload release
                    // [...]
                    // ReleasesMap.set(releaseLink.assetId, uploadedId)
                }                
            }
        }
    }
}

// Save files metadata
async function saveAssets(foundAssets){

    const success = [];
    for (const foundAsset of foundAssets){
       // foundAsset.asset.metadata - file metadata
       // [...]
       success.push(foundAsset)
    }

    return success
}

// Do submit
async function submitAssets (savedAssets) {
    if (savedAssets.length === 0) return [];

    const success = [];
    for (const savedAsset of savedAssets){
       // [...]
       success.push(savedAsset)
    }

    return success
}

async function processAssets (assets) {
    if (assets.length === 0){
        return;
    }
    
    // Find uploaded files
    const foundAssets = await findAssets(assets)
    if (foundAssets.length === 0){
        return
    }

    // Upload releases
    await uploadReleases(foundAssets);

    // Save metadata
    const savedAssets = await saveAssets(foundAssets)

    // Do submit
    let doneAssets;
    if (submitContext.settings.clickSubmit){
        doneAssets = await submitAssets(savedAssets);
    }
    else doneAssets = savedAssets

    // Mark, what is done
    for (const doneAsset of doneAssets){
        const mid = doneAsset.stockFile.id 
        doneAsset.asset.markDone({
            mid // save microstock file id in "submit" status
        });
    }
}

const auth = await checkAuth();
if (!auth){
    // Mark all files, that user is not authorized
    for (const asset of assets){
        asset.markUnauthorized();
    }
}
else {
    // Process files
    await processAssets(assets);
}